#!/usr/bin/ruby -w
#
# cache - Cache support
#
# version: $Id: cache.rb 213 2004-02-07 06:54:19Z kapheine $
#
# Copyright (C) 2003 Zachary P. Landau <kapheine@hypa.net>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

require 'musicextras/config'
require 'musicextras/utils'
require 'ftools'
require 'timeout'
require 'yaml'

module MusicExtras

  ### Handles caching of data
  class Cache
    # Stores the directory where the caches are stored
    attr_reader :dir

    # Department of redundancy department?
    GREYLIST_FILE = File.join($config['cache_dir'], 'greylist.lst')

    # Should exist whenever greylist is being accessed
    GREYLIST_LOCK = File.join($config['cache_dir'], '.greylist.lock')

    # [+dir+] the directory to store cache data, defaults to $config['cache_dir']
    def initialize(dir=$config['cache_dir'])
      @dir = dir

      File.makedirs($config['cache_dir']) 
      setup_greylist() unless defined? @@greylist
    end

    ### Reads the greylist from the greylist file or creates a new one
    def setup_greylist()
      if File.exists?(GREYLIST_FILE)
	greylist_lock do
	  File.open(GREYLIST_FILE, 'r') do |f|
	    @@greylist = YAML.load(f)
	  end
	end
      else
	@@greylist = Array.new
      end
    end

    ### Handles greylist locking for the given block
    ### [+block+] Code that needs exclusive access to the greylist
    def greylist_lock(&block)
	begin
	  timeout(10) do
	    while File.exists?(GREYLIST_LOCK)
	      sleep(0.25)
	    end
	  end
	rescue Timeout::Error
	  # 10 seconds to read the file? doubtful. we'll continue.
	end

	File.open(GREYLIST_LOCK, 'w')
	block.call
	File.delete(GREYLIST_LOCK)
    end

    # Saves data in the cache, returning the file it was saved as
    # [+key+] What to save the cache as. Should be a directory structure, but
    #         all non-word chars will be removed, except the /s. For example:
    #         Tom Petty/Free Fallin' will be converted to TomPetty/FreeFallin
    # [+data+] The data to store
    def save(key, data)
      file = get_filename(key)

      File.makedirs(File.dirname(file))
      File.open(file, 'w') do |f|
	f.write(data)
      end

      return file
    end

    # Loads data from the cache, returns the data, Cache::GREYLISTED, if it's
    # on the greylist, or nil, if it isn't on the list
    #
    # [+key+] What to save the cache as. Should be a directory structure, but
    #         all non-word chars will be removed, except the /s. For example:
    #         Tom Petty/Free Fallin' will be converted to TomPetty/FreeFallin
    def load(key)
      file = get_filename(key)

      if File.exists?(file)
	return File.new(file).read
      else
	return nil
      end

    end

    # Adds a key to the greylist
    # [+key+] The key to add to the list.
    def add_to_greylist(key)
      file = get_filename(key, false)
      @@greylist << file

      greylist_lock do 
	File.open(GREYLIST_FILE, 'w') do |f|
	  YAML.dump(@@greylist.sort, f)
	end
      end
    end

    # Checks to see if a key is in the greylist
    def greylisted?(key)
      return @@greylist.include?(get_filename(key, false))
    end

    # Returns the filename for a given key
    # [+key+] The key to get the filename from
    # [+include_path+] If true, will include full path to file. Defaults to true.
    def get_filename(key, include_path=true)

      # Make sure to preserve extension, if there is one
      if key.index('.')
	base = key.split('.')[0..-2].to_s
	extension = '.' + key.split('.')[-1]
      else
	base = key
	extension = ''
      end

      path = File.basename(base, extension)

      # Split up dirs, mangle their names, and put them back together
      dirs = []
      base.split('/').each do |dir|
	dirs << dir.mangle
      end
      full_dir = dirs.join('/') + "#{extension}"

      if include_path
	return File.join(@dir, full_dir)
      else
	return full_dir
      end
    end

  end

end
