#!/usr/bin/ruby -w
#
# gtkgui - Gtk interface for musicextras
#
# version: $Id: gtkgui.rb 206 2004-01-29 18:38:11Z kapheine $
#
# Copyright (C) 2003 Zachary P. Landau <kapheine@hypa.net>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

require 'musicextras/config'
require 'musicextras/gui/gui'
require 'musicextras/utils'
require 'libglade2'
require 'tmpdir'

module MusicExtras

  ### Gtk Gui interface to musicextras
  ### See Gui documentation for more information
  class GtkGui < Gui

    def initialize(control)
      super(control)

      Gtk.init
      @glade = GladeXML.new(File.find_in_path('musicextras/gui/musicextras.glade', $LOAD_PATH)) { |handler| method(handler)}

      @image_w = $config['image_w'] || 150
      @image_h = $config['image_h'] || 150

      main_window = @glade.get_widget('main_window')
      main_window.resize($config['window_w'], $config['window_h'])
      main_window.show

      @album_cover_set = @artist_image_set = nil

      # Set some stuff up for the incremental searching
      buffer = @glade.get_widget('lyrics_text').buffer
      @search_buffer = nil
      buffer.place_cursor(buffer.start_iter)
      @search_end_iter = buffer.start_iter
    end

    def start
      Gtk.main
    end

    def stop
      Gtk.main_quit
      Gtk.main_iteration	    # to make quit effective? why is this needed?
    end

    ### For now this just sets the title and (if given) artist 
    def set_song(title, artist=nil, album=nil)
      g_title = title || ''
      g_artist = artist || ''
      g_album = album || ''

      @glade.get_widget('main_window').set_title("#{artist}: #{title}")
    end

    def set_lyrics(lyrics)
      @glade.get_widget('lyrics_text').buffer.set_text(clean_text(lyrics))
    end

    def set_biography(bio)
      @glade.get_widget('bio_text').buffer.set_text(clean_text(bio))
    end

    def set_years_active(years)
      @glade.get_widget('years_active_text').set_text(years)
    end

    def set_album_cover(cover)
      # XXX: Would be nice to use cover data directly instead of having to write
      #      to a file, but I think you need gdkimlib's create_image_from_data,
      #      which is not implemented yet

      album_image = @glade.get_widget('album_image')
      images_box = @glade.get_widget('images_box')

      if cover.nil?
	@album_cover_set = false
	unless @album_cover_set or @artist_image_set
	  images_box.hide
	end
	album_image.hide
      else
	@album_cover_set = true
	if @album_cover_set or @artist_image_set
	  images_box.show
	end
	album_image.show
	file = write_image_to_file(cover, 'musicextras_cover.img')
	begin
	  pixbuf = Gdk::Pixbuf.new(file)
	  scale_w = ( ( pixbuf.width.to_f / pixbuf.height.to_f ) * @image_h).round
	  scale_w = pixbuf.width if scale_w > 2.5*@image_h
	  pixbuf = pixbuf.scale(scale_w, @image_h)
	  album_image.set(pixbuf)
	rescue RuntimeError
	  # probably couldnt recognize the file type, call ourselves with nil
	  set_album_cover(nil)
	end
      end
    end

    def set_artist_image(image)
      artist_image = @glade.get_widget('artist_image')
      images_box = @glade.get_widget('images_box')

      if image.nil?
	@artist_image_set = false
	unless @album_cover_set or @artist_image_set
	  images_box.hide
	end
	artist_image.hide
      else
	@artist_image_set = true
	if @album_cover_set or @artist_image_set
	  images_box.show
	end
	artist_image.show
	file = write_image_to_file(image, 'musicextras_artist.img')
	begin
	  pixbuf = Gdk::Pixbuf.new(file)
	  scale_w = ( ( pixbuf.width.to_f / pixbuf.height.to_f ) * @image_h).round
	  scale_w = pixbuf.width if scale_w > 2.5*@image_h
	  pixbuf = pixbuf.scale(scale_w, @image_h)
	  artist_image.set(pixbuf)
	rescue RuntimeError
	  set_artist_image(nil)
	end
      end
    end

    def set_status(msg)
      @glade.get_widget('statusbar').pop(1)
      @glade.get_widget('statusbar').push(1, msg)
    end

    private

    ### When close button is clicked...
    def on_close_button_clicked
      @control.close_clicked
    end

    ### When extra info close button is clicked..
    def on_extra_close_clicked
      @glade.get_widget('extra_window').hide
    end

    def on_lyrics_click(w, event)
      if event.button == 3
	menu = @glade.get_widget('right_click_menu')
	menu.popup(nil, nil, event.button, event.time)
      end
    end

    def on_edit_activate
      @control.edit_lyrics
    end

    def on_more_information_activate
      @glade.get_widget('extra_window').show
    end


    ### Called when a key is pressed anywhere in window
    ### [window] the GtkWindow the key was pressed in
    ### [event} the GdkEvent created
    def on_key_press(window, event)
      # Warning: don't look at this code. in case of accidentally viewing,
      # induce vomiting. just kidding, you will vomit anyway.
      
      buffer = @glade.get_widget('lyrics_text').buffer
      statusbar = @glade.get_widget('statusbar')

      if event.keyval == 65307 # ESC
	@search_buffer = nil
	buffer.place_cursor(buffer.start_iter)
	statusbar.pop(1)

      elsif event.keyval == 47 # /
	if @search_buffer.nil?
	  @search_buffer = ""

	  # Put cursor back at beginning of text
	  buffer.place_cursor(buffer.start_iter)
	  @search_end_iter = buffer.start_iter

	  statusbar.push(1, "Incremental search, start typing")
	end

      elsif event.keyval == 103 and ((event.state & Gdk::Window::CONTROL_MASK) != 0) # g

	if @search_buffer
	  buffer.place_cursor(@search_end_iter) if @search_end_iter
	  @search_end_iter = search_text(@search_buffer)
	end

      elsif event.keyval.between?(1, 255) # Any other chars
	unless @search_buffer.nil?
	  if (@search_end_iter = search_text(@search_buffer+event.keyval.chr))
	    @search_buffer += event.keyval.chr
	    statusbar.pop(1)
	    statusbar.push(1, "Search: #{@search_buffer}") if @search_end_iter
	  end
	end

      end

    end

    ### Searches for given text in lyrics box. Highlights the text if found.
    ### Returns a GtkTextIter pointing to the end of the search term, or nil
    ### if nothing is found.
    ### [txt] a String to search for
    ### [wrap] Will wrap search if true. Defaults to true.
    def search_text(txt, wrap=true)
      end_iter = nil
      buffer = @glade.get_widget('lyrics_text').buffer
      iter = buffer.get_iter_at_mark(buffer.get_mark('insert'))

      found = iter.forward_search(txt, 0, nil)
      if !found and wrap
	buffer.place_cursor(buffer.start_iter)
	found = iter.forward_search(txt, 0, nil)
      end

      if found
	buffer.place_cursor(found[0])
	buffer.move_mark('selection_bound', found[-1])
	end_iter = found[-1]
      end

      end_iter

=begin 
      ### XXX: not working yet
      
      if found
	@prev_search_iters = found
	#return @prev_search_iters[-1]
	return found[-1]
      elsif @prev_search_iters
	#buffer.place_cursor(@prev_search_iters[0])
	#buffer.move_mark('selection_bound', @prev_search_iters[-1])
	return @prev_search_iters[-1]
      end

      end_iter
=end

    end

    ### Writes a string containing an image to a temporary file, returns the
    ### full filename
    ### [image] a Binary String containing the image
    ### [filename] filename to use in Dir.tmpdir
    def write_image_to_file(image, filename)
      file = File.join(Dir.tmpdir, filename)

      File.open(file, 'w') do |f|
	f.write(image)
      end

      return file
    end

    ### Removes chars that GTK can't display, returning the clean text
    ### [+text+] The text to clean
    def clean_text(text)
      text.gsub(/[^\s\w\.\"\'\,\-\!\(\)@]/, '') # remove bad chars
    end
  end

end
