#!/usr/bin/ruby -w
#
# guicontrol - Controller for GUI
#
# version: $Id: guicontrol.rb 221 2004-02-08 19:48:56Z kapheine $
#
# Copyright (C) 2003 Zachary P. Landau <kapheine@hypa.net>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

require 'musicextras/config'
require 'musicextras/utils'
require 'socket'
require 'rexml/document'
require 'rexml/streamlistener'
require 'base64'

autoload :GtkGui, 'musicextras/gui/gtkgui'

# :stopdoc:
module REXML
  class IOSource
    alias_method :_initialize, :initialize

    def initialize(arg, blocksize=500)
      @er_source = @source = arg
      @to_utf = false
      @line_break = '>'
      super @source.readline(@line_break)
      @line_break = encode('>')
    end
  end
end
# :startdoc:

module MusicExtras

  ### Provides interface to guis
  class GuiControl
    include REXML::StreamListener

    # GUI instance
    attr_reader :gui

    ### Creates the gui control
    ### [+guiclass+] The class of the Gui to load. Defaults to GtkGui
    def initialize(guiclass=GtkGui)
      @gui = guiclass.new(self)
      @title = @artist = @album = nil
      @lyrics_file = nil

      @clients = Array.new
      create_input_thread()
    end

    ### Start up the Gui
    def start
      @gui.start
    end

    ### when the Close button is clicked...
    def close_clicked
      @gui.stop
    end

    ### Loads up an editor for the current lyrics file
    def edit_lyrics
      system($config['editor'] % @lyrics_file) if @lyrics_file
    end

    ### Sets the current message on the statusbar
    ### [+msg+] The message to display
    def set_status(msg)
      @gui.set_status(msg)
    end

    ### Starts up a thread with a tcp server. The thread will wait for
    ### for incoming XML. When it receives some, it will pass
    ### the necessary method to update the gui information
    def create_input_thread # :nodoc:
      Thread.new do
	server = TCPServer.new(*$config['gui_host'].split(':'))

	while (session = server.accept)
	  Thread.new do
	    @clients << session

	    check_clients()
	    doc = REXML::Document.parse_stream(session, self)

	    @clients.delete(session)
	  end
	end

      end
    end

    def tag_start(item, attrs) # :nodoc:
      @current_tag = item
      @text = ''

      if attrs['encoded']
	@decode = true
      else
	@decode = false
      end
    end

    def text(text) # :nodoc:
      @text = text.to_s
    end

    def tag_end(item) # :nodoc:
      tag = @current_tag
      return unless tag
      data = @text

      if @decode == true
	data = decode64(data)
      end

      case tag
      when 'lyrics'
	@gui.set_lyrics(data)
      when 'lyrics_file'
	@lyrics_file = data
      when 'status'
	@gui.set_status(data)
      when 'artist_image'
	@gui.set_artist_image(data == '' ? nil : data)
      when 'album_cover'
	@gui.set_album_cover(data == '' ? nil : data)
      when 'title'
	@title = data
	unless @artist.nil?
	  @gui.set_song(@title, @artist)
	  @title = @artist = nil
	end
      when 'artist'
	@artist = data
	unless @title.nil?
	  @gui.set_song(@title, @artist)
	  @title = @artist = nil
	end
      when 'album'
      when 'biography'
	@gui.set_biography(data)
      when 'years_active'
	@gui.set_years_active(data)
      else
      end

      @current_tag = nil
    end

    private

    ### Checks for multiple clients. All except the latest are disconnected.
    ### (Well really, the socket is just closed, and the client is left to 
    ### figure out he has been disconnected)
    def check_clients
      unless @clients.size == 1
	@clients[0..-2].each do |c|
	  # close the stream on them. they should catch the error and
	  # assume we disconnected them. yeah it's dirty.
	  c.close
	  @clients.delete(c)
	end
      end
    end

  end

end
