#!/usr/bin/ruby -w
#
# musicsite - Abstract MusicSite class
#
# version: $Id: musicsite.rb 222 2004-02-08 22:45:20Z kapheine $
#
# Copyright (C) 2003 Zachary P. Landau <kapheine@hypa.net>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

require 'musicextras/config'
require 'musicextras/utils'
require 'digest/md5'
require 'open-uri'
require 'ftools'
require 'logger'
require 'singleton'

module MusicExtras

  # == Writing musicsite plugins
  #
  # Adding new site plugins should be fairly straight forward. I'll use parts of
  # the AllMusic plugin as an example. 
  #
  #   class AllMusic < MusicSite
  #
  #   def initialize
  #     super('AllMusic', 'www.allmusic.com')
  #
  #     Artist::register_plugin(self, :image, '{ARTIST}/{ARTIST}.img')
  #     Album::register_plugin(self, :cover, '{ARTIST}/covers/{ALBUM}.img')
  #   end
  #
  # The first parameter to super should be a meaningful name for the plugin
  # (without spaces). The second parameter should be the base url for the site.
  #
  # The next two lines register a method with an aggregator. So once the
  # first line is registered, the image method in your plugin will be called
  # when artist.image is called. So your image method should return the binary
  # data for the image. The third line is where the information is cached on
  # the filesystem. See the Artist.cache_values, Album.cache_values, and
  # Song.cache_values for the available {} variables. You may make up any
  # cache path you want, but it should make sense, and stay consistent with
  # the other plugins.
  #
  #   MusicExtras::AllMusic.new
  #
  # At the end of the plugin, a line similar to the one above should appear.
  # At this point the plugin will register itself. To have your plugin get
  # loaded automatically, add it to musicextras/musicsites/_load_sites.rb
  #
  # Use the available plugins as an example. There are some helper functions
  # available to you from inheriting this class so check out the code below.
  #
  class MusicSite
    # Stores the name of the plugin
    attr_reader :name

    # Stores the url of page for the plugin
    attr_reader :url

    # Useragents you can use if your plugin needs it
    USERAGENTS = {
      'Mozilla' => 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.2.1) Gecko/20030225',
      'Opera'   => 'Opera/6.0 (Linux 2.4.18 i686; U)  [en]',
      'Lynx'      => 'Lynx/2.8.4dev.16 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.6-beta3'
    }

    ### Creates a new music site. All plugins should call super so this is ran.
    ### [+name+] A meaningful name for the site (without spaces)
    ### [+url+] The base url of the site (eg: www.allmusic.com)
    def initialize(name, url)
      @name = name
      @url = url

      @logger = Log.instance
    end

    ### Sends a message to the logger
    ### [+msg+] The message string to log
    def info(msg)
      @logger.info(@name, msg)
    end

    ### Prints in the following format: 'MusicSite: (url)'
    def to_s
      "#{@name}: #{@url}"
    end

    ### Compares two strings using String.mangle
    ### [+str1+] the first String
    ### [+str2+] the second String
    ### [+extra+] additional Regex to apply to match. Defaults to //.
    ### Returns true or false
    def match?(str1, str2, extra=//)
      return str1.mangle(extra) == str2.mangle(extra)
    end

    ### Extracts text from an html page and returns it without the html tags.
    ### [+page+] A String containing the html
    ### [+regexp+] A Regexp with only one (), that should contain the text
    ###            Make sure to include /m at the end for multiple lines
    def extract_text(page, regexp)
      begin
	page.scan(regexp) do |line|
	  return line.to_s.strip_html
	end
      rescue
	return nil
      end

      info("extract_text: regexp failed")
      return nil
    end

    CACHE_LIFE = 60*60*24*7

    ### Downloads a page, given a url. Returns the contents, or nil if
    ### there was a problem. Uses a 7 day file cache.
    ### [+url+] url to download (@url should contain the hostname)
    ### [+useragent+] optional useragent string to pass to server
    ### [+extra+] Optional header data to pass to server
    def fetch_page(url, agent="", *extra)
      if (url =~ /^http:/)
	uri = url
      else
	uri = "http://#{@url}#{url}"
      end

      params = Array.new
      params.push("User-Agent" => agent)
      params.concat(extra)

      @page_cache = File.join($config['cache_dir'], 'webcache')
      File.makedirs(@page_cache) unless File.exists?(@page_cache)
      cache_file = File.join(@page_cache, Digest::MD5.hexdigest(uri))
    
      begin

	if File.exists?(cache_file) and
	  Time.now - File.mtime(cache_file) < CACHE_LIFE 
	  data = File.open(cache_file).read
	else
	  open(uri, *params) do |page|
	    data = page.read
	    File.open(cache_file, "w") { |f| f.write(data) }
	  end
	end
      rescue => error
	info("fetch_page: error retrieving #{uri}: #{error}")
      end

      data
    end
  
    ### This is a wrapper used by site plugins to log status information
    class Log
      include Singleton

      def initialize
	if $config['log']
	  File.makedirs(File.dirname($config['logdev']))
	  @log = Logger.new($config['logdev'], 1, 200000)
	  @log.datetime_format = "%Y-%m-%d %H:%M:%S"
	end
      end

      ### Logs a message
      ### [+plugin+] The name of the plugin creating the log
      ### [+msg+] The Message to log
      def info(plugin, msg)
	@log.info(plugin) { msg } if $config['log']
      end
    end

  end

end
