#!/usr/bin/ruby18 -w
#
# pusher - handles command line arguments, retrieving data, and sending
#	   it out
#
# version: $Id: pusher.rb 218 2004-02-07 23:11:56Z kapheine $
#
# Copyright (C) 2003 Zachary P. Landau <kapheine@hypa.net>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

require 'musicextras/musicsites/_load_sites'
require 'musicextras/album'
require 'musicextras/artist'
require 'musicextras/song'
require 'musicextras/cache'
require 'musicextras/guicontrol'
require 'base64'
require 'getoptlong'
require 'rexml/document'

module MusicExtras
  Version = ''

  ### This class handles input from the user, retrieves the data,
  ### and sends it off to the GUI.
  class Pusher
    PROGNAME = File.basename($0)

    ### Parses the command line arguments
    def initialize
      parse_options()
      @output = STDOUT
    end

    ### Retrieves and outputs the data
    def output
      begin
	@output = TCPSocket.new(*$config['gui_host'].split(':')) if @options['gui']

	xml_header()
	retrieve_data()
	xml_footer()
      rescue Errno::EPIPE
	# Another client probably connected, so just silently quit
      end
    end

    private

    ### Display version information
    def version()
      puts "#{PROGNAME} #{Version}"
      puts "Copyright (C) 2004 Zachary P. Landau"
    end

    ### Display the usage information
    def usage()
      version()
      puts ""
      puts "usage: #{PROGNAME} <options>"
      puts "-a, --artist\tspecify artist\n" +
	"-l, --album\tspecify album\n" +
	"-t, --title\tspecify song title\n" +
	"-f, --file=FILE\tspecify information from mp3 tag\n" +
	"\n" +
	"-g, --gui\tdisplay information using gui (default)\n" +
	"-s, --stdout\tdisplay information to stdout\n" +
	"-q, --quiet\tdon't display notices\n" +
	"\n" +
	"-i, --noimages\tdon't fetch images\n" +
	"\n" +
	"-c, --clear\tclears the greylist and exits\n" +
	"\n" +
	"-h, --help\tthis usage display\n"
    end

    ### Parse the options, returning a hash of options
    def parse_options
      @options = {}
      @song = nil

      opts = GetoptLong.new(
	['--artist', '-a', GetoptLong::REQUIRED_ARGUMENT],
	['--album', '-l', GetoptLong::REQUIRED_ARGUMENT],
	['--title', '-t', GetoptLong::REQUIRED_ARGUMENT],
	['--file', '-f', GetoptLong::REQUIRED_ARGUMENT],
	['--gui', '-g', GetoptLong::NO_ARGUMENT],
	['--stdout', '-s', GetoptLong::NO_ARGUMENT],
	['--quiet', '-q', GetoptLong::NO_ARGUMENT],
	['--noimages', '-i', GetoptLong::NO_ARGUMENT],
	['--clear', '-c', GetoptLong::NO_ARGUMENT],
	['--help', '-h', GetoptLong::NO_ARGUMENT],
	['--version', '-V', GetoptLong::NO_ARGUMENT]
       )

      # set defaults
      @options['gui'] = true
      @options['quiet'] = $config['quiet'] || false

      begin
	opts.each do |opt, arg|
	  case opt
	  when '--artist' || '-a'
	    @options['artist'] = arg
	  when '--album' || '-l'
	    @options['album'] = arg
	  when '--title' || '-t'
	    @options['title'] = arg
	  when '--file' || '-f'
	    @song = Song.from_mp3_tag(arg)
	    @options['title'] = @song.title
	    @options['artist'] = @song.artist.name
	    @options['album'] = @song.album.title
	  when '--gui' || '-g'
	    @options['gui'] = true
	  when '--stdout' || '-s'
	    @options['gui'] = false
	    @output = STDOUT
	  when '--noimages' || 'i'
	    @options['noimages'] = true
	  when '--quiet' || '-q'
	    @options['quiet'] = true
	  when '--clear' || '-c'
	    info("Clearing the greylist in #{Cache::GREYLIST_FILE}.")
	    clear_greylist()
	    exit
	  when '--help' || '-h'
	    usage()
	    exit 0
	  when '--version' || '-V'
	    version()
	    exit 0
	  end
	end
      rescue GetoptLong::InvalidOption => msg
	exit 1
      end

      @options
    end

    ### Uses the command line options to create appropriate Song, Album, Artist,
    ### etc. Sends the tags to the queue as they are retrieved
    def retrieve_data
      # XXX: yeah, so, fuck threads. if i run this shit in a thread, the program just
      # quits somewhere in song.lyrics. no error or anything. so fuck threads.

      clear_old_tags if @options['gui']

      artist_pre_regex if @options['artist']
      handle_lyrics
      handle_artist_image unless @options['noimages']
      handle_album_cover unless @options['noimages']
      handle_biography if $config['more_info']
      handle_years_active if $config['more_info']

      output_tag('STATUS', '') if @options['gui']
    end

    # Send empty tags to clear out old info
    def clear_old_tags
      output_tag('LYRICS', '')
      output_tag('LYRICS_FILE', '')
      output_tag('ARTIST_IMAGE', nil)
      output_tag('ALBUM_COVER', nil)
      output_tag('TITLE', '')
      output_tag('ARTIST', '')
      output_tag('ALBUM', '')
      output_tag('BIOGRAPHY', '')
      output_tag('YEARS_ACTIVE', '')
      output_tag('STATUS', '')
    end

    ### Applies $config['artist_pre_regex'] regex to @options['artist']
    def artist_pre_regex
      regexp_list(@options['artist'], $config['artist_pre_regex']) do |a|
	@options['artist'] = a
      end
    end

    ### Takes an array of [regexp, sub] pairs to apply to a string, yielding
    ### each one
    ### [+str+] The string to apply the regexp to
    ### [+list+] List of [regexp, sub] pairs
    def regexp_list(str, list)
      list.each do |regex|
	sub = str.sub(regex[0], regex[1])
	yield sub if sub != str
      end
    end

    ### Sends LYRICS => Lyrics_Data tag to queue if lyrics are found,
    ### nil otherwise
    def handle_lyrics
      output_tag('STATUS', 'Retrieving lyrics..') if @options['gui']

      unless @song
	if @options['title'] && @options['artist']
	  @song = Song.new(@options['title'], @options['artist'])
	else
	  info('Notice: Not retrieving lyrics (missing title or artist)')
	  return
	end
      end

      if @song
	begin
	  lyrics = @song.lyrics

	  # If lyrics aren't found, start applying regex to the artist
	  unless lyrics
	    regexp_list(@options['artist'], $config['artist_cond_regex'] ) do |a|
	      lyrics = Song.new(@options['title'], a).lyrics
	    end
	  end
	rescue DataAccessor::AccessorNotImplemented
	  lyrics = 'No lyrics plugin loaded.'
	end
	lyrics = 'No lyrics found.' if lyrics.nil?
      end

      send_song_info

      output_tag('LYRICS', lyrics)
      output_tag('LYRICS_FILE', @song.cached_as(:lyrics))
    end

    ### Sends ARTIST_IMAGE => Binary_Data tag to queue if image is found,
    ### nil otherwise
    def handle_artist_image
      output_tag('STATUS', 'Retrieving artist image..') if @options['gui']

      if @options['artist']
	artist = Artist.new(@options['artist'])
	begin
	  image = artist.image

	  unless image
	    regexp_list(@options['artist'], $config['artist_cond_regex'] ) do |a|
	      image = Artist.new(a).image
	    end
	  end

	rescue DataAccessor::AccessorNotImplemented
	  image = nil
	end
      else
	info('Notice: Not retrieving artist image (missing artist)')
	image = nil
      end

      output_tag('ARTIST_IMAGE', image, true)
    end

    ### Sends ALBUM_COVER => Binary_Data tag to queue if cover is found,
    ### nil otherwise
    def handle_album_cover
      output_tag('STATUS', 'Retrieving album cover..') if @options['gui']

      if @options['album'] && @options['artist']
	album = Album.new(@options['album'], @options['artist'])

	begin
	  cover = album.cover

	  unless cover
	    regexp_list(@options['artist'], $config['artist_cond_regex'] ) do |a|
	      cover = Album.new(@options['album'], a).cover
	    end
	  end

	rescue DataAccessor::AccessorNotImplemented
	  cover = nil
	end
      else
	info('Notice: Not retrieving album cover (missing artist or album)')
	cover = nil
      end

      output_tag('ALBUM_COVER', cover, true)
    end

    ### Sends BIOGRAPHY => Bio_Data if found
    def handle_biography
      output_tag('STATUS', 'Retrieving biography..') if @options['gui']

      if @options['artist'] 
	artist = Artist.new(@options['artist'])

	begin
	  bio = artist.biography

	  unless bio
	    regexp_list(@options['artist'], $config['artist_cond_regex']) do |a|
	      bio = Artist.new(a).biography
	    end
	  end
	rescue DataAccessor::AccessorNotImplemented
	  bio = nil
	end
      else
	info('Notice: not retrieving biography (missing artist)')
	bio = nil
      end

      output_tag('BIOGRAPHY', bio, false)
    end

    ### Sends YEARS_ACTIVE => data if found
    def handle_years_active
      output_tag('STATUS', 'Retrieving years active..') if @options['gui']

      if @options['artist']
	artist = Artist.new(@options['artist'])

	begin
	  years = artist.years_active

	  unless years
	    regexp_list(@options['artist'], $config['artist_cond_regex']) do |a|
	      years = Artist.new(a).years_active
	    end
	  end

	rescue DataAccessor::AccessorNotImpelemented
	  years = nil
	end
      else
	info('Notice: not retrieving years active (missing artist)')
	years = nil
      end

      output_tag('YEARS_ACTIVE', years, false)
    end

    ### Sends any song information that we have
    ### - title, artist, album
    def send_song_info
      output_tag('TITLE', @options['title']) if @options['title']
      output_tag('ARTIST', @options['artist']) if @options['artist']
      output_tag('ALBUM', @options['album']) if @options['album']
    end

    ### Deletes the greylist file, if it exists
    def clear_greylist()
      File.unlink(Cache::GREYLIST_FILE) rescue Errno::ENOENT
    end

    ### Outputs the tag to $output
    ### [key] The key for the data
    ### [data] The data associated with the key
    ### [binary] Set to true if it is binary data, false is default
    def output_tag(key, data, binary=false)
      data = '' unless data

      element = REXML::Element.new(key.downcase)
      element.text = binary ? encode64(data) : data
      element.attributes['encoded'] = 'base64' if binary

      @output.puts "\t" + element.to_s
    end

    ### Output the XML file header
    def xml_header
      @output.puts "<?xml version='1.0'?>"
      @output.puts '<musicextras>'
    end

    ### Output the XML file footer
    def xml_footer
      @output.puts '</musicextras>'
    end

    ### Displays string on STDOUT unless quiet is set
    ### [+msg+] The message to display
    def info(msg)
      puts(msg) unless @options['quiet']
    end

  end
end
